=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2011-05-03 22:53:39 +0000
+++ bzrlib/bzrdir.py	2011-05-03 23:16:56 +0000
@@ -1252,15 +1252,12 @@
     def has_workingtree(self):
         """Tell if this bzrdir contains a working tree.
 
-        This will still raise an exception if the bzrdir has a workingtree that
-        is remote & inaccessible.
-
         Note: if you're going to open the working tree, you should just go
         ahead and try, and not ask permission first.
         """
         from bzrlib.workingtree import WorkingTreeFormat
         try:
-            WorkingTreeFormat.find_format(self)
+            WorkingTreeFormat.find_format_string(self)
         except errors.NoWorkingTree:
             return False
         return True

=== modified file 'bzrlib/tests/test_import_tariff.py'
--- bzrlib/tests/test_import_tariff.py	2011-05-03 23:07:04 +0000
+++ bzrlib/tests/test_import_tariff.py	2011-05-03 23:16:56 +0000
@@ -207,6 +207,8 @@
             'bzrlib.bundle.commands',
             'bzrlib.cmd_version_info',
             'bzrlib.dirstate',
+            'bzrlib._dirstate_helpers_py',
+            'bzrlib._dirstate_helpers_pyx',
             'bzrlib.externalcommand',
             'bzrlib.filters',
             # foreign branch plugins import the foreign_vcs_registry from 

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2011-05-03 22:53:39 +0000
+++ bzrlib/tests/test_workingtree.py	2011-05-03 23:16:56 +0000
@@ -166,6 +166,19 @@
 class TestWorkingTreeFormat(TestCaseWithTransport):
     """Tests for the WorkingTreeFormat facility."""
 
+    def test_find_format_string(self):
+        # is the right format object found for a working tree?
+        branch = self.make_branch('branch')
+        self.assertRaises(errors.NoWorkingTree,
+            workingtree.WorkingTreeFormat.find_format_string, branch.bzrdir)
+        transport = branch.bzrdir.get_workingtree_transport(None)
+        transport.mkdir('.')
+        transport.put_bytes("format", "some format name")
+        # The format does not have to be known by Bazaar,
+        # find_format_string just retrieves the name
+        self.assertEquals("some format name",
+            workingtree.WorkingTreeFormat.find_format_string(branch.bzrdir))
+
     def test_find_format(self):
         # is the right format object found for a working tree?
         # create a branch with a few known format objects.

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2011-05-03 22:53:39 +0000
+++ bzrlib/workingtree.py	2011-05-03 23:16:56 +0000
@@ -3019,14 +3019,20 @@
     """If this format supports missing parent conflicts."""
 
     @classmethod
+    def find_format_string(klass, a_bzrdir):
+        """Return format name for the working tree object in a_bzrdir."""
+        try:
+            transport = a_bzrdir.get_workingtree_transport(None)
+            return transport.get_bytes("format")
+        except errors.NoSuchFile:
+            raise errors.NoWorkingTree(base=transport.base)
+
+    @classmethod
     def find_format(klass, a_bzrdir):
         """Return the format for the working tree object in a_bzrdir."""
         try:
-            transport = a_bzrdir.get_workingtree_transport(None)
-            format_string = transport.get_bytes("format")
+            format_string = klass.find_format_string(a_bzrdir)
             return format_registry.get(format_string)
-        except errors.NoSuchFile:
-            raise errors.NoWorkingTree(base=transport.base)
         except KeyError:
             raise errors.UnknownFormatError(format=format_string,
                                             kind="working tree")

